@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, Yue Du
+ * Copyright (c) 2014-2016, Yue Du
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification,
@@ -35,6 +35,7 @@
 #define TOSTRING(x) #x
 #define VALUE_TO_STRING(x) TOSTRING(x)
 
+#define XXHASH_VERSION XXH_VERSION_MAJOR.XXH_VERSION_MINOR.XXH_VERSION_RELEASE
 #define XXH32_DIGESTSIZE 4
 #define XXH32_BLOCKSIZE 16
 #define XXH64_DIGESTSIZE 8
@@ -84,7 +85,7 @@ static void ull2bytes(unsigned long long in, char *out)
 typedef struct {
     PyObject_HEAD
     /* Type-specific fields go here. */
-    XXH32_state_t xxhash_state[1];
+    XXH32_state_t *xxhash_state;
     unsigned int seed;
 } PYXXH32Object;
 
@@ -92,6 +93,7 @@ static PyTypeObject PYXXH32Type;
 
 static void PYXXH32_dealloc(PYXXH32Object *self)
 {
+    XXH32_freeState(self->xxhash_state);
     PyObject_Del(self);
 }
 
@@ -105,6 +107,10 @@ static PyObject *PYXXH32_new(PyTypeObject *type, PyObject *args, PyObject *kwarg
         return NULL;
     }
 
+    if ((self->xxhash_state = XXH32_createState()) == NULL) {
+        return NULL;
+    }
+
     return (PyObject *)self;
 }
 
@@ -131,8 +137,8 @@ static int PYXXH32_init(PYXXH32Object *self, PyObject *args, PyObject *kwargs)
 
 PyDoc_STRVAR(
     PYXXH32_update_doc,
-    "update (arg)\n\n"
-    "Update the xxh32 object with the string arg. Repeated calls are\n"
+    "update (input)\n\n"
+    "Update the xxh32 object with the string input. Repeated calls are\n"
     "equivalent to a single call with the concatenation of all the arguments.");
 
 static PyObject *PYXXH32_update(PYXXH32Object *self, PyObject *args)
@@ -269,8 +275,12 @@ static PyObject *PYXXH32_copy(PYXXH32Object *self)
         return NULL;
     }
 
+    if ((p->xxhash_state = XXH32_createState()) == NULL) {
+        return NULL;
+    }
+
     p->seed = self->seed;
-    memcpy(p->xxhash_state, self->xxhash_state, sizeof(self->xxhash_state));
+    XXH32_copyState(p->xxhash_state, self->xxhash_state);
 
     return (PyObject *)p;
 }
@@ -364,7 +374,7 @@ PyDoc_STRVAR(
     "\n"
     "Methods:\n"
     "\n"
-    "update() -- updates the current digest with an additional string\n"
+    "update(input) -- updates the current digest with the provided string.\n"
     "digest() -- return the current digest value\n"
     "hexdigest() -- return the current digest as a string of hexadecimal digits\n"
     "intdigest() -- return the current digest as an integer\n"
@@ -422,7 +432,7 @@ static PyTypeObject PYXXH32Type = {
 typedef struct {
     PyObject_HEAD
     /* Type-specific fields go here. */
-    XXH64_state_t xxhash_state[1];
+    XXH64_state_t *xxhash_state;
     unsigned long long seed;
 } PYXXH64Object;
 
@@ -430,6 +440,7 @@ static PyTypeObject PYXXH64Type;
 
 static void PYXXH64_dealloc(PYXXH64Object *self)
 {
+    XXH64_freeState(self->xxhash_state);
     PyObject_Del(self);
 }
 
@@ -441,6 +452,10 @@ static PyObject *PYXXH64_new(PyTypeObject *type, PyObject *args, PyObject *kwarg
         return NULL;
     }
 
+    if ((self->xxhash_state = XXH64_createState()) == NULL) {
+        return NULL;
+    }
+
     return (PyObject *)self;
 }
 
@@ -467,8 +482,8 @@ static int PYXXH64_init(PYXXH64Object *self, PyObject *args, PyObject *kwargs)
 
 PyDoc_STRVAR(
     PYXXH64_update_doc,
-    "update (arg)\n\n"
-    "Update the xxh64 object with the string arg. Repeated calls are\n"
+    "update (input)\n\n"
+    "Update the xxh64 object with the string input. Repeated calls are\n"
     "equivalent to a single call with the concatenation of all the arguments.");
 
 static PyObject *PYXXH64_update(PYXXH64Object *self, PyObject *args)
@@ -605,8 +620,12 @@ static PyObject *PYXXH64_copy(PYXXH64Object *self)
         return NULL;
     }
 
+    if ((p->xxhash_state = XXH64_createState()) == NULL) {
+        return NULL;
+    }
+
     p->seed = self->seed;
-    memcpy(p->xxhash_state, self->xxhash_state, sizeof(self->xxhash_state));
+    XXH64_copyState(p->xxhash_state, self->xxhash_state);
 
     return (PyObject *)p;
 }
@@ -700,7 +719,7 @@ PyDoc_STRVAR(
     "\n"
     "Methods:\n"
     "\n"
-    "update() -- updates the current digest with an additional string\n"
+    "update(input) -- updates the current digest with an additional string\n"
     "digest() -- return the current digest value\n"
     "hexdigest() -- return the current digest as a string of hexadecimal digits\n"
     "intdigest() -- return the current digest as an integer\n"
@@ -833,8 +852,6 @@ void initxxhash(void)
         INITERROR;
     }
 
-    PYXXH32Type.tp_new = PyType_GenericNew;
-
     if (PyType_Ready(&PYXXH32Type) < 0) {
         INITERROR;
     }
@@ -843,8 +860,6 @@ void initxxhash(void)
     PyModule_AddObject(module, "xxh32", (PyObject *)&PYXXH32Type);
 
 
-    PYXXH64Type.tp_new = PyType_GenericNew;
-
     if (PyType_Ready(&PYXXH64Type) < 0) {
         INITERROR;
     }
